# from typing import Any, Callable, Dict

# from openai import AsyncAzureOpenAI, AzureOpenAI, OpenAI
# from openai.types.chat.chat_completion_chunk import (
#     ChatCompletionChunk,
#     Choice as ChoiceChunk,
#     ChoiceDelta,
# )

# from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
# from llm_adapters.types import (
#     AdapterChatCompletionChunk,
#     Conversation,
#     ConversationRole,
#     Cost,
#     Model,
#     ModelProperties,
#     Provider,
#     Vendor,
# )


# class AzureModel(Model):
#     provider_name: str = Provider.azure.value

#     properties: ModelProperties = ModelProperties(gdpr_compliant=True)


# MODELS: list[Model] = [
#     AzureModel(
#         name="gpt-4o",
#         cost=Cost(prompt=5.0e-6, completion=15.0e-6),
#         context_length=128000,
#         completion_length=4096,
#         vendor_name=Vendor.openai.value,
#     ),
#     AzureModel(
#         name="gpt-4o-mini",
#         cost=Cost(prompt=0.15e-6, completion=0.6e-6),
#         context_length=128000,
#         completion_length=16385,
#         vendor_name=Vendor.openai.value,
#     ),
# ]


# class AzureSDKChatProviderAdapter(OpenAISDKChatAdapter):
#     @staticmethod
#     def get_supported_models() -> list[Model]:
#         return MODELS

#     @staticmethod
#     def get_api_key_name() -> str:
#         return "AZURE_API_KEY"

#     def get_base_sdk_url(self) -> str:
#         return "https://martiantest.openai.azure.com/"

#     def _call_sync(self) -> Callable[..., Any]:
#         return self._client_sync.chat.completions.create

#     def _call_async(self) -> Callable[..., Any]:
#         return self._client_async.chat.completions.create

#     def _create_client_sync(self, base_url: str, api_key: str) -> OpenAI:
#         return AzureOpenAI(
#             api_key=api_key,
#             azure_endpoint=base_url,
#             api_version="2024-06-01",
#         )

#     def _create_client_async(self, base_url: str, api_key: str) -> AsyncAzureOpenAI:
#         return AsyncAzureOpenAI(
#             api_key=api_key,
#             azure_endpoint=base_url,
#             api_version="2024-06-01",
#         )

#     def _get_params(self, llm_input: Conversation, **kwargs: Any) -> Dict[str, Any]:
#         params = super()._get_params(llm_input, **kwargs)

#         azure_tool_choice = kwargs.get("tool_choice")

#         if azure_tool_choice == "required":
#             azure_tool_choice = "auto"

#         return {
#             **params,
#             "tool_choice": azure_tool_choice,
#         }

#     def _extract_stream_response(
#         self, request: Any, response: ChatCompletionChunk, state: dict[str, Any]
#     ) -> AdapterChatCompletionChunk:
#         adapter_response = AdapterChatCompletionChunk.model_construct(
#             **response.model_dump(),
#         )

#         if len(adapter_response.choices) == 0:
#             adapter_response.choices = [
#                 ChoiceChunk(
#                     index=0,
#                     delta=ChoiceDelta(
#                         role=ConversationRole.assistant.value, content=""
#                     ),
#                 )
#             ]

#         return adapter_response



# from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
# from llm_adapters.types import Cost, Model, Provider, Vendor


# class OctoaiModel(Model):
#     provider_name: str = Provider.octoai.value

#     supports_vision: bool = False
#     supports_tools: bool = False


# MODELS: list[Model] = [
#     OctoaiModel(
#         name="hermes-2-pro-llama-3-8b",
#         cost=Cost(prompt=0.15e-6, completion=0.15e-6),
#         context_length=8192,
#         vendor_name=Vendor.hermes_llama.value,
#     ),
#     OctoaiModel(
#         name="meta-llama-3-70b-instruct",
#         cost=Cost(prompt=0.9e-6, completion=0.9e-6),
#         context_length=8192,
#         vendor_name=Vendor.meta_llama.value,
#     ),
#     OctoaiModel(
#         name="nous-hermes-2-mixtral-8x7b-dpo",
#         cost=Cost(prompt=0.45e-6, completion=0.45e-6),
#         context_length=8192,
#         vendor_name=Vendor.nous_hermes.value,
#     ),
#     OctoaiModel(
#         name="mixtral-8x7b-instruct",
#         cost=Cost(prompt=0.45e-6, completion=0.45e-6),
#         context_length=32768,
#         vendor_name=Vendor.mixtral.value,
#     ),
# ]


# class OctoaiSDKChatProviderAdapter(OpenAISDKChatAdapter):
#     @staticmethod
#     def get_supported_models() -> list[Model]:
#         return MODELS

#     @staticmethod
#     def get_api_key_name() -> str:
#         return "OCTOAI_API_KEY"

#     def get_base_sdk_url(self) -> str:
#         return "https://text.octoai.run/v1"



# from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
# from llm_adapters.types import Cost, Model, Provider, Vendor


# class TensorOperaModel(Model):
#     provider_name: str = Provider.tensoropera.value
#     vendor_name: str = Vendor.moescape.value

#     supports_completion: bool = True
#     supports_chat: bool = False
#     supports_max_completion_tokens: bool = False


# MODELS: list[Model] = [
#     TensorOperaModel(
#         name="euryale",
#         api_name="tensoropera-yodayo/L3.1-70B-Euryale-v2.2_gpu4",
#         cost=Cost(prompt=0, completion=0),
#         context_length=12288,
#         can_system_only=False,
#     ),
# ]


# class TensorOperaSDKChatProviderAdapter(OpenAISDKChatAdapter):
#     @staticmethod
#     def get_supported_models() -> list[Model]:
#         return MODELS

#     @staticmethod
#     def get_api_key_name() -> str:
#         return "TENSOROPERA_API_KEY"

#     def get_base_sdk_url(self) -> str:
#         return "https://open.tensoropera.ai/inference/api/v1"



from typing import Any, Dict, Literal


# TODO: Add time based expiration if leaks connections or memory
class ClientCache:
    def __init__(self) -> None:
        self._client_cache: Dict[str, Any] = {}

    def get_client(
        self, base_url: str, api_key: str, mode: Literal["sync", "async"]
    ) -> Any:
        return self._client_cache.get(f"{base_url}-{api_key}-{mode}")

    def set_client(
        self, base_url: str, api_key: str, mode: Literal["sync", "async"], client: Any
    ) -> None:
        self._client_cache[f"{base_url}-{api_key}-{mode}"] = client


client_cache = ClientCache()



# from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
# from llm_adapters.types import Cost, Model, Provider, Vendor


# class FireworksModel(Model):
#     provider_name: str = Provider.fireworks.value

#     supports_vision: bool = False
#     can_min_p: bool = False
#     can_top_k: bool = False

#     def get_api_path(self) -> str:
#         if self.name == "yi-large":
#             return f"accounts/yi-01-ai/models/{self.name}"

#         return f"accounts/fireworks/models/{self.name}"


# # TODO: Add more models
# MODELS: list[Model] = [
#     FireworksModel(
#         name="llama-v3p1-405b-instruct",
#         cost=Cost(prompt=3.00e-6, completion=3.00e-6),
#         context_length=131072,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3p1-70b-instruct",
#         cost=Cost(prompt=0.90e-6, completion=0.90e-6),
#         context_length=131072,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3p1-8b-instruct",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=131072,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3p2-3b-instruct",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=131072,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3p2-11b-vision-instruct",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=131072,
#         vendor_name=Vendor.meta_llama.value,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3p2-90b-vision-instruct",
#         cost=Cost(prompt=0.90e-6, completion=0.90e-6),
#         context_length=131072,
#         vendor_name=Vendor.meta_llama.value,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="yi-large",
#         cost=Cost(prompt=3.00e-6, completion=3.00e-6),
#         context_length=32768,
#         vendor_name=Vendor.O1.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3-70b-instruct-hf",
#         cost=Cost(prompt=0.90e-6, completion=0.90e-6),
#         context_length=8192,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3-70b-instruct",
#         cost=Cost(prompt=0.90e-6, completion=0.90e-6),
#         context_length=8192,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3-8b-instruct-hf",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=8192,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="llama-v3-8b-instruct",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=8192,
#         vendor_name=Vendor.meta_llama.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="mythomax-l2-13b",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=4096,
#         vendor_name=Vendor.gryphe.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
#     FireworksModel(
#         name="qwen2p5-72b-instruct",
#         cost=Cost(prompt=0.90e-6, completion=0.90e-6),
#         context_length=32768,
#         vendor_name=Vendor.qwen.value,
#         supports_vision=False,
#         supports_tools=False,
#     ),
# ]


# class FireworksSDKChatProviderAdapter(OpenAISDKChatAdapter):
#     @staticmethod
#     def get_supported_models() -> list[Model]:
#         return MODELS

#     @staticmethod
#     def get_api_key_name() -> str:
#         return "FIREWORKS_API_KEY"

#     def get_base_sdk_url(self) -> str:
#         return "https://api.fireworks.ai/inference/v1"

#     # def _get_params(
#     #     self, messages: list[ChatCompletionMessageParam], **kwargs: Any
#     # ) -> dict[str, Any]:
#     #     params = super()._get_params(messages, **kwargs)

#     #     # Keep only last image_url for vision
#     #     # skiped_image = False
#     #     # for message in reversed(params["messages"]):
#     #     #     if isinstance(message["content"], list):
#     #     #         for content in reversed(message["content"]):
#     #     #             if content["type"] == "image_url":
#     #     #                 if skiped_image:
#     #     #                     content["type"] = "text"
#     #     #                     content["text"] = content["image_url"]["url"]
#     #     #                     del content["image_url"]
#     #     #                 else:
#     #     #                     skiped_image = True

#     #     # # Remove image details
#     #     # for message in params["messages"]:
#     #     #     if isinstance(message["content"], list):
#     #     #         for content in message["content"]:
#     #     #             if content["type"] == "image_url":
#     #     #                 del content["image_url"]["details"]

#     #     return params



# from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
# from llm_adapters.general_utils import YUAN_TO_USD
# from llm_adapters.types import (
#     Cost,
#     Model,
#     ModelProperties,
#     Provider,
#     Vendor,
# )


# class BigModelModel(Model):
#     provider_name: str = Provider.bigmodel.value
#     vendor_name: str = Vendor.bigmodel.value

#     supports_completion: bool = False
#     supports_vision: bool = False
#     supports_tools: bool = False
#     supports_tool_choice: bool = False
#     supports_tool_choice_required: bool = False
#     supports_n: bool = False
#     supports_stop: bool = False

#     can_empty_content: bool = False
#     can_system_only: bool = False
#     can_assistant_only: bool = False
#     can_min_p: bool = False
#     can_top_k: bool = False

#     properties: ModelProperties = ModelProperties(
#         gdpr_compliant=False,
#     )


# # TODO: check completion length
# MODELS: list[Model] = [
#     BigModelModel(
#         name="glm-4-plus",
#         cost=Cost(prompt=0.00005e-6 * YUAN_TO_USD, completion=0.00005e-6 * YUAN_TO_USD),
#         context_length=128000,
#         completion_length=4096,
#     ),
#     BigModelModel(
#         name="glm-4-air-0111",
#         cost=Cost(
#             prompt=0.0000005e-6 * YUAN_TO_USD, completion=0.0000005e-6 * YUAN_TO_USD
#         ),
#         context_length=128000,
#         completion_length=4096,
#     ),
#     BigModelModel(
#         name="glm-4-long",
#         cost=Cost(
#             prompt=0.000001e-6 * YUAN_TO_USD, completion=0.000001e-6 * YUAN_TO_USD
#         ),
#         context_length=1000000,
#         completion_length=4096,
#     ),
#     BigModelModel(
#         name="glm-zero-preview",
#         cost=Cost(prompt=0.00001e-6 * YUAN_TO_USD, completion=0.00001e-6 * YUAN_TO_USD),
#         context_length=16000,
#         completion_length=4096,
#     ),
#     BigModelModel(
#         name="glm-4-airx",
#         cost=Cost(prompt=0.00001e-6 * YUAN_TO_USD, completion=0.00001e-6 * YUAN_TO_USD),
#         context_length=8000,
#         completion_length=4096,
#     ),
#     BigModelModel(
#         name="glm-4-flashx",
#         cost=Cost(
#             prompt=0.0000001e-6 * YUAN_TO_USD, completion=0.0000001e-6 * YUAN_TO_USD
#         ),
#         context_length=128000,
#         completion_length=4096,
#     ),
# ]


# class BigModelSDKChatProviderAdapter(OpenAISDKChatAdapter):
#     @staticmethod
#     def get_supported_models() -> list[Model]:
#         return MODELS

#     @staticmethod
#     def get_api_key_name() -> str:
#         return "BIGMODEL_API_KEY"

#     def get_base_sdk_url(self) -> str:
#         return "https://open.bigmodel.cn/api/paas/v4"



# from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
# from llm_adapters.types import Model, Provider


# class OpenRouterModel(Model):
#     provider_name: str = Provider.openrouter.value

#     def get_api_path(self) -> str:
#         return f"{self.vendor_name}/{self.name}"


# MODELS: list[Model] = []


# class OpenRouterSDKChatProviderAdapter(OpenAISDKChatAdapter):
#     @staticmethod
#     def get_supported_models() -> list[Model]:
#         return MODELS

#     @staticmethod
#     def get_api_key_name() -> str:
#         return "OPENROUTER_API_KEY"

#     def get_base_sdk_url(self) -> str:
#         return "https://openrouter.ai/api/v1"



from llm_adapters import general_utils
from llm_adapters.abstract_adapters import (
    ApiKeyAdapterMixin,
    Adapter,
    OpenAISDKChatAdapter,
    SDKChatAdapter,
    ProviderAdapterMixin,
)
from llm_adapters.types import (
    Cost,
    Model,
    AdapterChatCompletion,
    AdapterStreamChatCompletion,
    AdapterStreamSyncChatCompletion,
    AdapterStreamAsyncChatCompletion,
    AdapterChatCompletionChunk,
    AdapterException,
    AdapterRateLimitException,
    AdapterModelNotFoundException,
    AdapterCompletion,
    AdapterCompletionChunk,
    AdapterStreamCompletion,
    AdapterStreamSyncCompletion,
    AdapterStreamAsyncCompletion,
    Provider,
    Vendor,
    ConversationRole,
    AdapterFinishReason,
    ContentType,
    ModelProperties,
)
from llm_adapters.adapter_factory import AdapterFactory
from llm_adapters.client import OpenAI, AsyncOpenAI

__all__ = [
    "OpenAI",
    "AsyncOpenAI",
    "general_utils",
    "ApiKeyAdapterMixin",
    "Adapter",
    "OpenAISDKChatAdapter",
    "SDKChatAdapter",
    "ProviderAdapterMixin",
    "TurnType",
    "ContentTurn",
    "FunctionOutputTurn",
    "ToolOutputTurn",
    "ToolsCallTurn",
    "FunctionCallTurn",
    "Cost",
    "Model",
    "AdapterChatCompletion",
    "AdapterStreamChatCompletion",
    "AdapterStreamSyncChatCompletion",
    "AdapterStreamAsyncChatCompletion",
    "AdapterChatCompletionChunk",
    "AdapterException",
    "AdapterRateLimitException",
    "AdapterModelNotFoundException",
    "Prompt",
    "AdapterResponse",
    "AdapterStreamResponse",
    "AdapterCompletion",
    "AdapterCompletionChunk",
    "AdapterStreamCompletion",
    "AdapterStreamSyncCompletion",
    "AdapterStreamAsyncCompletion",
    "Provider",
    "Vendor",
    "ConversationRole",
    "AdapterFinishReason",
    "VisionImageDetails",
    "TextContentEntry",
    "ImageContentEntry",
    "ContentType",
    "ModelProperties",
    "AdapterFactory",
]



from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
from llm_adapters.types import Cost, Model, Provider, Vendor


class CerebrasModel(Model):
    provider_name: str = Provider.cerebras.value

    supports_n: bool = False
    supports_vision: bool = False

    can_min_p: bool = False
    can_top_k: bool = False


MODELS: list[Model] = [
    CerebrasModel(
        name="llama3.1-8b",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=0.10e-6, completion=0.10e-6),
        context_length=32768,
    ),
    CerebrasModel(
        name="llama3.3-70b",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=0.85e-6, completion=1.20e-6),
        context_length=32768,
    ),
    # CerebrasModel(
    #     name="deepseek-r1-distill-llama-70b",
    #     vendor_name=Vendor.deepseek.value,
    #     cost=Cost(prompt=0.85e-6, completion=1.20e-6),
    #     context_length=65536,
    # ),
]


class CerebrasSDKChatProviderAdapter(OpenAISDKChatAdapter):
    @staticmethod
    def get_supported_models() -> list[Model]:
        return MODELS

    @staticmethod
    def get_api_key_name() -> str:
        return "CEREBRAS_API_KEY"

    def get_base_sdk_url(self) -> str:
        return "https://api.cerebras.ai/v1"



from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
from llm_adapters.types import Model, Provider, Vendor


# An average token corresponds to 1 word or 6 characters of English text.
# Comparatively, this is up to 30% more text per token than other providers, saving you 30% of the cost.
class AI21Model(Model):
    provider_name: str = Provider.ai21.value
    vendor_name: str = Vendor.ai21.value

    supports_tools: bool = False
    supports_completion: bool = False
    supports_json_content: bool = False
    supports_vision: bool = False
    supports_streaming: bool = False

    can_system_only: bool = False
    can_empty_content: bool = False
    can_min_p: bool = False
    can_top_k: bool = False


MODELS: list[Model] = [
    # AI21Model(
    #     name="jamba-1.6-mini",
    #     cost=Cost(prompt=0.2e-6, completion=0.4e-6),
    #     context_length=256000,
    # ),
    # AI21Model(
    #     name="jamba-1.6-large",
    #     cost=Cost(prompt=2.0e-6, completion=8.0e-6),
    #     context_length=256000,
    # ),
]


class AI21SDKChatProviderAdapter(OpenAISDKChatAdapter):
    @staticmethod
    def get_supported_models() -> list[Model]:
        return MODELS

    @staticmethod
    def get_api_key_name() -> str:
        return "AI21_API_KEY"

    def get_base_sdk_url(self) -> str:
        return "https://api.ai21.com/studio/v1"



# from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
# from llm_adapters.types import Cost, Model, Provider, Vendor


# class GroqModel(Model):
#     provider_name: str = Provider.groq.value

#     supports_vision: bool = False
#     supports_tools: bool = False


# MODELS: list[Model] = [
#     GroqModel(
#         name="llama-3.1-70b-versatile",
#         cost=Cost(prompt=0.59e-6, completion=0.79e-6),
#         context_length=131072,
#         vendor_name="meta-llama",
#     ),
#     GroqModel(
#         name="llama-3.1-8b-instant",
#         cost=Cost(prompt=0.05e-6, completion=0.08e-6),
#         context_length=131072,
#         vendor_name="meta-llama",
#     ),
#     GroqModel(
#         name="llama3-70b-8192",
#         cost=Cost(prompt=0.59e-6, completion=0.79e-6),
#         context_length=8192,
#         vendor_name="meta-llama",
#     ),
#     GroqModel(
#         name="llama3-8b-8192",
#         cost=Cost(prompt=0.05e-6, completion=0.08e-6),
#         context_length=8192,
#         vendor_name="meta-llama",
#     ),
#     GroqModel(
#         name="mixtral-8x7b-32768",
#         cost=Cost(prompt=0.24e-6, completion=0.24e-6),
#         context_length=32768,
#         vendor_name="mistralai",
#     ),
#     GroqModel(
#         name="gemma-7b-it",
#         cost=Cost(prompt=0.07e-6, completion=0.07e-6),
#         context_length=8192,
#         vendor_name=Vendor.google.value,
#     ),
#     GroqModel(
#         name="gemma2-9b-it",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=8192,
#         vendor_name="google",
#     ),
#     GroqModel(
#         name="llama3-groq-8b-8192-tool-use-preview",
#         cost=Cost(prompt=0.19e-6, completion=0.19e-6),
#         context_length=8192,
#         vendor_name="groq",
#     ),
#     GroqModel(
#         name="llama-guard-3-8b",
#         cost=Cost(prompt=0.20e-6, completion=0.20e-6),
#         context_length=8192,
#         vendor_name="meta-llama",
#     ),
# ]


# class GroqSDKChatProviderAdapter(OpenAISDKChatAdapter):
#     @staticmethod
#     def get_supported_models() -> list[Model]:
#         return MODELS

#     @staticmethod
#     def get_api_key_name() -> str:
#         return "GROQ_API_KEY"

#     def get_base_sdk_url(self) -> str:
#         return "https://api.groq.com/openai/v1"



from llm_adapters.abstract_adapters.openai_sdk_chat_adapter import OpenAISDKChatAdapter
from llm_adapters.types import (
    Cost,
    Model,
    Provider,
    Vendor,
)


class TogetherModel(Model):
    provider_name: str = Provider.together.value

    supports_tools: bool = False
    supports_completion: bool = False
    supports_n: bool = False
    supports_vision: bool = False

    can_min_p: bool = False
    can_top_k: bool = False

    def get_api_path(self) -> str:
        return f"{self.vendor_name}/{self.name}"


MODELS: list[Model] = [
    TogetherModel(
        name="DeepSeek-R1",
        vendor_name=Vendor.deepseek_ai.value,
        cost=Cost(prompt=3.00e-6, completion=7.00e-6),
        context_length=164000,
    ),
    TogetherModel(
        name="DeepSeek-V3",
        vendor_name=Vendor.deepseek_ai.value,
        cost=Cost(prompt=1.25e-6, completion=1.25e-6),
        context_length=131000,
        can_assistant_only=False,
    ),
    TogetherModel(
        name="Qwen2.5-72B-Instruct-Turbo",
        vendor_name=Vendor.Qwen.value,
        cost=Cost(prompt=1.20e-6, completion=1.20e-6),
        context_length=32768,
    ),
    TogetherModel(
        name="Mistral-Small-24B-Instruct-2501",
        vendor_name=Vendor.mistralai.value,
        cost=Cost(prompt=0.80e-6, completion=0.80e-6),
        context_length=32768,
        can_assistant_only=False,
    ),
    TogetherModel(
        name="Llama-3.1-Nemotron-70B-Instruct-HF",
        vendor_name=Vendor.nvidia.value,
        cost=Cost(prompt=0.88e-6, completion=0.88e-6),
        context_length=128000,
    ),
    TogetherModel(
        name="Llama-3.3-70B-Instruct-Turbo",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=0.88e-6, completion=0.88e-6),
        context_length=128000,
        supports_json_content=False,
    ),
    TogetherModel(
        name="Meta-Llama-3.1-405B-Instruct-Turbo",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=3.50e-6, completion=3.50e-6),
        context_length=131000,
        supports_json_content=False,
    ),
    TogetherModel(
        name="Qwen2.5-Coder-32B-Instruct",
        vendor_name=Vendor.Qwen.value,
        cost=Cost(prompt=0.80e-6, completion=0.80e-6),
        context_length=32768,
    ),
    TogetherModel(
        name="gemma-2-27b-it",
        vendor_name=Vendor.google.value,
        cost=Cost(prompt=0.80e-6, completion=0.80e-6),
        context_length=8000,
    ),
    # TogetherModel(
    #     name="DeepSeek-R1-Distill-Llama-70B",
    #     vendor_name=Vendor.deepseek_ai.value,
    #     cost=Cost(prompt=2.00e-6, completion=2.00e-6),
    #     context_length=128000,
    # ),
    # TogetherModel(
    #     name="DeepSeek-R1-Distill-Qwen-14",
    #     vendor_name=Vendor.deepseek_ai.value,
    #     cost=Cost(prompt=1.60e-6, completion=1.60e-6),
    #     context_length=128000,
    # ),
    TogetherModel(
        name="DeepSeek-R1-Distill-Qwen-1.5B",
        vendor_name=Vendor.deepseek_ai.value,
        cost=Cost(prompt=0.18e-6, completion=0.18e-6),
        context_length=128000,
    ),
    TogetherModel(
        name="Llama-4-Maverick-17B-128E-Instruct-FP8",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=0.27e-6, completion=0.85e-6),
        context_length=500000,
        supports_vision=True,
        can_empty_content=False,
    ),
    TogetherModel(
        name="Llama-4-Scout-17B-16E-Instruct",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=0.18e-6, completion=0.59e-6),
        context_length=300000,
        supports_vision=True,
        can_empty_content=False,
    ),
    # TogetherModel(
    #     name="Llama-3.3-70B-Instruct-Turbo-Free",
    #     vendor_name=Vendor.meta_llama.value,
    #     cost=Cost(prompt=0, completion=0),  # Free model
    #     context_length=8000,
    #     supports_json_content=False,
    # ),
    # TogetherModel(
    #     name="Llama-Vision-Free",
    #     vendor_name=Vendor.meta_llama.value,
    #     cost=Cost(prompt=0, completion=0),  # Free model
    #     context_length=128000,
    #     supports_vision=True,
    #     supports_json_output=False,
    #     can_empty_content=False,
    # ),
    TogetherModel(
        name="Llama-3.2-11B-Vision-Instruct-Turbo",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=0.18e-6, completion=0.18e-6),
        context_length=128000,
        supports_vision=True,
        supports_json_output=False,
        can_empty_content=False,
    ),
    TogetherModel(
        name="Llama-3.2-90B-Vision-Instruct-Turbo",
        vendor_name=Vendor.meta_llama.value,
        cost=Cost(prompt=1.20e-6, completion=1.20e-6),
        context_length=128000,
        supports_vision=True,
    ),
]

DEFAULT_TEMPERATURE = 0.7


class TogetherSDKChatProviderAdapter(OpenAISDKChatAdapter):
    @staticmethod
    def get_supported_models() -> list[Model]:
        return MODELS

    @staticmethod
    def get_api_key_name() -> str:
        return "TOGETHER_API_KEY"

    def get_base_sdk_url(self) -> str:
        return "https://api.together.xyz"

    def _adjust_temperature(self, temperature: float) -> float:
        return temperature / 2

    # def _get_params(
    #     self, messages: list[ChatCompletionMessageParam], **kwargs: Any
    # ) -> dict[str, Any]:
    #     params = super()._get_params(messages, **kwargs)

    #     # If the user has requested n messages, but not specified a temperature, we need to provide default temperature
    #     if params.get("n") and params.get("temperature") is None:
    #         params["temperature"] = DEFAULT_TEMPERATURE

    #     # Keep only last image_url for vision
    #     skiped_image = False
    #     for message in reversed(params["messages"]):
    #         if isinstance(message["content"], list):
    #             for content in reversed(message["content"]):
    #                 if content["type"] == "image_url":
    #                     if skiped_image:
    #                         content["type"] = "text"
    #                         content["text"] = content["image_url"]["url"]
    #                         del content["image_url"]
    #                     else:
    #                         skiped_image = True

    #     return params



from abc import abstractmethod
import os
from typing import List


class ApiKeyAdapterMixin:
    _api_key: str = ""
    _api_keys: List[str] = []
    _next_api_key = 0

    def __init__(self) -> None:
        api_keys = os.environ.get(f"{self.get_api_key_name()}_LIST", None)
        if api_keys:
            ApiKeyAdapterMixin._api_keys = api_keys.split(",")
        else:
            api_key = os.environ.get(self.get_api_key_name())
            self._api_keys = [api_key] if api_key else []

    @staticmethod
    @abstractmethod
    def get_api_key_name() -> str:
        """returns the api key name for the adapter

        Returns:
            str: api key name for the adapter (e.g. "OPENAI_API_KEY")
        """

    def _api_key_was_set_by_user(self) -> bool:
        return bool(self._api_key)

    def set_api_key(self, api_key: str) -> None:
        """sets an api key for the adapter

        Args:
            api_key: api key for the adapter
        """
        if not api_key:
            raise ValueError("api_key cannot be empty")
        self._api_key = api_key

    def get_api_key(self) -> str:
        """gets an api key for the adapter

        Returns:
            str: api key for the adapter if it exists, else the env default api key
        """

        if not self._api_key_was_set_by_user() and self._api_keys:
            api_key = self._api_keys[self._next_api_key]
            self._next_api_key += 1
            if len(self._api_keys) == self._next_api_key:
                self._next_api_key = 0
            return api_key
        return self._api_key



import os

from llm_adapters.abstract_adapters import Adapter
from llm_adapters.abstract_adapters.provider_adapter_mixin import ProviderAdapterMixin

from llm_adapters.provider_adapters import (
    AnthropicSDKChatProviderAdapter,
    GeminiSDKChatProviderAdapter,
    AI21SDKChatProviderAdapter,
    LambdaLabsSDKChatProviderAdapter,
    OpenAISDKChatProviderAdapter,
    CerebrasSDKChatProviderAdapter,
    CohereSDKChatProviderAdapter,
    DeepInfraSDKChatProviderAdapter,
    TogetherSDKChatProviderAdapter,
    DeepSeekSDKChatProviderAdapter,
)
from llm_adapters.types import Model, AdapterModelNotFoundException

ADAPTER_CLASSES: list[type[Adapter]] = [
    AnthropicSDKChatProviderAdapter,
    GeminiSDKChatProviderAdapter,
    AI21SDKChatProviderAdapter,
    LambdaLabsSDKChatProviderAdapter,
    OpenAISDKChatProviderAdapter,
    TogetherSDKChatProviderAdapter,
    DeepSeekSDKChatProviderAdapter,
    CerebrasSDKChatProviderAdapter,
    CohereSDKChatProviderAdapter,
    DeepInfraSDKChatProviderAdapter,
]


class AdapterFactory:
    @staticmethod
    def _create_adapter_registry() -> dict[str, type[Adapter]]:
        adapters_classes: dict[str, type[Adapter]] = {}

        for adapter_class in ADAPTER_CLASSES:
            if issubclass(adapter_class, ProviderAdapterMixin):
                for model in adapter_class.get_supported_models():
                    adapters_classes[model.get_path()] = adapter_class
                    adapters_classes[model.name] = adapter_class

        return adapters_classes

    @staticmethod
    def _create_model_registry() -> dict[str, Model]:
        models: dict[str, Model] = {}

        for adapter_class in ADAPTER_CLASSES:
            if issubclass(adapter_class, ProviderAdapterMixin):
                for model in adapter_class.get_supported_models():
                    models[model.get_path()] = model
                    models[model.name] = model

        return models

    @staticmethod
    def _create_model_list() -> list[Model]:
        models: list[Model] = []

        for adapter_class in ADAPTER_CLASSES:
            if issubclass(adapter_class, ProviderAdapterMixin):
                for model in adapter_class.get_supported_models():
                    models.append(model)

        return models

    _adapter_registry = _create_adapter_registry()

    _model_registry = _create_model_registry()
    _model_list = _create_model_list()

    @staticmethod
    def get_adapter_by_path(model_path: str) -> Adapter:
        adapter_class = AdapterFactory._adapter_registry.get(model_path)
        model = AdapterFactory._model_registry.get(model_path)

        if adapter_class is None or model is None:
            raise AdapterModelNotFoundException()

        adapter = adapter_class()

        if isinstance(adapter, ProviderAdapterMixin):
            adapter._set_current_model(model)

        return adapter

    @staticmethod
    def get_adapter(model: Model) -> Adapter:
        adapter_class = AdapterFactory._adapter_registry.get(model.get_path())

        if adapter_class is None:
            raise AdapterModelNotFoundException()

        adapter = adapter_class()

        if isinstance(adapter, ProviderAdapterMixin):
            adapter._set_current_model(model)

        return adapter

    @staticmethod
    def get_model_by_path(model_path: str) -> Model:
        model = AdapterFactory._model_registry.get(model_path)

        if model is None:
            raise AdapterModelNotFoundException()

        return model

    # TODO: add more supports filtering
    @staticmethod
    def get_supported_models(
        supports_streaming: bool = False,
        supports_vision: bool = False,
        supports_functions: bool = False,
        supports_tools: bool = False,
        supports_n: bool = False,
        supports_json_output: bool = False,
        supports_json_content: bool = False,
    ) -> list[Model]:
        disabled_models = os.getenv("ADAPTER_DISABLED_MODELS", "").split(",")

        return [
            model
            for model in AdapterFactory._model_list
            if (not supports_streaming or model.supports_streaming)
            and (not supports_vision or model.supports_vision)
            and (not supports_functions or model.supports_functions)
            and (not supports_tools or model.supports_tools)
            and (not supports_n or model.supports_n)
            and (not supports_json_output or model.supports_json_output)
            and (not supports_json_content or model.supports_json_content)
            and model.name not in disabled_models
        ]


__all__ = ["AdapterFactory"]



from .api_key_adapter_mixin import ApiKeyAdapterMixin
from .adapter import Adapter
from .openai_sdk_chat_adapter import OpenAISDKChatAdapter
from .sdk_chat_adapter import SDKChatAdapter
from .provider_adapter_mixin import ProviderAdapterMixin

__all__ = [
    "ApiKeyAdapterMixin",
    "Adapter",
    "OpenAISDKChatAdapter",
    "SDKChatAdapter",
    "ProviderAdapterMixin",
]



from abc import abstractmethod
from typing import List, Optional

from llm_adapters.types import Model


class ProviderAdapterMixin:
    _current_model: Optional[Model] = None

    def _set_current_model(self, model: Model) -> None:
        self._current_model = model

    @staticmethod
    @abstractmethod
    def get_supported_models() -> List[Model]:
        pass
